import pandas as pd
import argparse
from flywheel_bids_tools.upload_bids import get_unequal_cells
from flywheel_bids_tools.utils import relist_item


def read_flywheel_csv(fpath, required_cols=['acquisition.id']):
    '''
    Read in a CSV and also ensure it's one of ours

    Input:
        fpath: path to the file
        required_cols: list of columns to ensure csv is a flywheel query
    Output:
        df: a pandas dataframe
    '''

    df = pd.read_csv(fpath)

    if not all(elem in df.columns.tolist() for elem in required_cols):
        raise Exception(("It doesn't look like this csv is correctly formatted",
        " for this flywheel editing process!"))
    df = df.sort_values(by=["acquisition.id", "acquisition.label"])
    return(df)


def main():

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-grouped", "--grouped-input",
        dest='grouped',
        help="The original grouped Flywheel query file",
        required=True
    )
    parser.add_argument(
        "-mod", "--modified-input",
        help="The modified grouped query file",
        dest="modified",
        required=True
    )
    parser.add_argument(
        "-orig", "--original-input",
        dest='original',
        help="The original ungrouped Flywheel query file",
        required=True
    )
    parser.add_argument(
        "-output", "--output-file",
        help="Path and name of the desired output file with grouped changes mapped to the full dataset",
        dest="output",
        required=True
    )

    args = parser.parse_args()

    # grouped edited df
    df_grouped_modified = read_flywheel_csv(args.modified)
    groups = relist_item(df_grouped_modified['groups'][0])

    # grouped unedited
    df_grouped = read_flywheel_csv(args.grouped)

    # original df
    df_original = read_flywheel_csv(args.original)

    # add groupings
    # add a group index
    df_original['group_id'] = float('nan')
    # group the file and store separate groups
    grouped_df = df_original.groupby(groups).groups

    # loop over groups and assign the index
    id = 1
    for name, df in grouped_df.items():

        index = df.to_list()
        df_original.loc[index, 'group_id'] = id
        id += 1
    # index the differences
    diff = get_unequal_cells(df_grouped, df_grouped_modified, provenance=True)

    changes = []

    for x in diff:

        key = df_grouped_modified.loc[x[0], 'group_id']
        val = (df_grouped_modified.columns[x[1]], df_grouped_modified.iloc[x[0], x[1]])
        changes.append((key, val))

    # loop through the differences and map them to the full dataset
    print("Applying the changes to the full dataset...")

    for change in changes:
        df_original.loc[df_original['group_id'] == change[0], change[1][0]] = change[1][1]

    df_original.drop(columns='group_id', inplace=True)
    df_original = df_original.sort_values(by=["acquisition.id", "acquisition.label"])
    df_original.to_csv(args.output, index=False)
    print("Done")


if __name__ == '__main__':
    main()



import pandas as pd
import argparse
from flywheel_bids_tools.utils import unlist_item


def read_flywheel_csv(fpath, required_cols=['acquisition.id']):
    '''
    Read in a CSV and also ensure it's one of ours

    Input:
        fpath: path to the file
        required_cols: list of columns to ensure csv is a flywheel query
    Output:
        df: a pandas dataframe
    '''

    df = pd.read_csv(fpath)

    if not all(elem in df.columns.tolist() for elem in required_cols):
        raise Exception(("It doesn't look like this csv is correctly formatted",
        " for this flywheel editing process!"))
    #drop_downs = ['classification_Measurement', 'classification_Intent',
        #'classification_Features']
    #df.loc[:, drop_downs] = df.loc[:, drop_downs].applymap(unlist_item)
    df = df.sort_values(by=["acquisition.id", "acquisition.label"])
    return(df)


def main():

    parser = argparse.ArgumentParser(description=("Use this tool to group a Flywheel query file by a column with common values."))
    parser.add_argument(
        "-input", "--input-file",
        dest='infile',
        help="Flywheel query file",
        required=True
    )
    parser.add_argument(
        "-output", "--grouped-output",
        help="The path and name of a grouped version of the output CSV of the query",
        dest="group_output",
        required=True
    )
    parser.add_argument(
        "-groups", "--groupings",
        nargs='+',
        dest='group',
        help="Columns to group unique rows by",
        required=True
    )
    args = parser.parse_args()

    # read in the file
    query_result = read_flywheel_csv(args.infile)

    # add a group index
    query_result['group_id'] = float('nan')
    # group the file and store separate groups
    grouped_df = query_result.groupby(args.group).groups

    # loop over groups and assign the index
    id = 1
    for name, df in grouped_df.items():

        index = df.to_list()
        query_result.loc[index, 'group_id'] = id
        id += 1

    # finally, drop duplicates
    query_result = query_result.drop_duplicates(args.group)
    query_result['groups'] = unlist_item(args.group)
    query_result = query_result.sort_values(by=["acquisition.id", "acquisition.label"])
    query_result.to_csv(args.group_output, index=False)
    print("Done")


if __name__ == '__main__':
    main()



import pandas as pd
import flywheel
import warnings
import argparse
from tabulate import tabulate
from pandas.io.json.normalize import nested_to_record
import json
import sys


def find_gear(gear_name, client):
    '''
    Queries flywheel for a gear by name
    '''

    try:
        gear = client.lookup('gears/{}'.format(gear_name))
        return gear
    except:
        print("No gear found by that name. Use \"query-gears -name all\" to see all the gears available to you")
        SystemExit(0)


def collect_gear_config(gear_id, client):
    '''
    Collects the gear's configuration and inputs
    '''
    gear = client.get_gear(gear_id)
    name = gear['gear']['name']
    label = gear['gear']['label']
    description = gear['gear']['description']
    inputs = nested_to_record(gear.gear.inputs)
    config = nested_to_record(gear.get_default_config())
    return({'name': name, 'inputs': inputs, 'config': config, 'label': label, 'description': description})


def str2bool(v):
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.')


def main():

    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        fw = flywheel.Client()
        assert fw, "Your Flywheel CLI credentials aren't set!"

    parser = argparse.ArgumentParser(description=("Use this to query Flywheel for the gears available to you, or get the config file for a gear."))

    parser.add_argument(
        "-name", "--gear-name",
        dest='name',
        help="Shorthand name of the gear on Flywheel",
        required=True,
        default='all'
    )
    parser.add_argument(
        "-config", "--output-config",
        dest='config',
        help="True/False; Whether to output configuration file for running",
        required=False,
        default='False'
    )

    args = parser.parse_args()

    config = str2bool(args.config)

    if args.name == 'all':
        gears = fw.gears()
        gears_table = [nested_to_record(g.to_dict(), sep='_') for g in gears]
        df = pd.DataFrame(gears_table)
        df = df.filter(regex=r'gear_label$|gear_name$|^category$', axis = 1)
        print(tabulate(df, headers='keys', tablefmt='psql',))

    else:
        gear = find_gear(args.name, fw)
        config_file = collect_gear_config(gear['_id'], fw)
        if config:
            with open('gear_config.json', 'w') as outfile:
                json.dump(config_file, outfile)
            print("Config file written.")
        else:
            print(json.dumps(config_file, indent=4))

if __name__ == '__main__':
    main()



class BidsGenerator:

    def __init__(self):
        self.dictionary = None
        self.valid = False
        self.subject = None
        self.session = None
        self.acq = None
        self.ce = None
        self.rec = None
        self.run = None
        self.mod = None
        self.task = None
        self.dir = None
        self.suffix = None


    def parse_row(self, pd_row):

        self.dictionary = pd_row.to_dict()
        self.subject = self.dictionary['subject.label']
        self.session = self.dictionary['session.label']



import json
import argparse
import flywheel
import warnings

def process_config(config):
    '''
    Process the config file to determine the inputs and outputs
    '''

    if config['name'] == "dicom-mr-classifier":

        input_1 = "acquisition.id"
    inputs = config['inputs']
    print(list(inputs.keys()))
    print('base' in list(inputs.keys()))


def main():

    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        fw = flywheel.Client()
        assert fw, "Your Flywheel CLI credentials aren't set!"

    parser = argparse.ArgumentParser(description=("Use this tool to run a Flywheel gear on the changes you've made."))

    parser.add_argument(
        "-config", "--config-file",
        dest='config',
        help="Path to the config.json file from a gear query",
        required=True
    )
    parser.add_argument(
        "-orig",
        help="Path to the original flywheel query CSV",
        dest="original",
        required=True
    )
    parser.add_argument(
        "-mod",
        help="Path to the modified flywheel query CSV",
        dest="modified",
        required=True
    )

    args = parser.parse_args()

    with open(args.config) as f:
        config = json.load(f)

    process_config(config)



import pandas as pd
import numpy as np
from datetime import datetime
import os


def unlist_item(ls):
    '''Convert a list item to a comma-separated string
    '''
    if isinstance(ls, list):
        ls.sort()
        return(', '.join(x for x in ls))
    else:
        return float('nan')


def relist_item(string):
    if type(string) is str:
        return([s.strip() for s in string.split(',')])
    else:
        return string


def read_flywheel_csv(fpath, required_cols=['acquisition.id']):
    '''
    Read in a CSV and also ensure it's one of ours

    Input:
        fpath: path to the file
        required_cols: list of columns to ensure csv is a flywheel query
    Output:
        df: a pandas dataframe
    '''

    df = pd.read_csv(fpath)

    if not all(elem in df.columns.tolist() for elem in required_cols):
        raise Exception(("It doesn't look like this csv is correctly formatted",
        " for this flywheel editing process!"))
    df = df.sort_values(by=["acquisition.id", "acquisition.label"])
    return(df)


def get_unequal_cells(df1, df2, provenance=True):
    '''
    Compare two dataframes and return indeces where the values are not equal

    Input:
    -------
        df1: original pandas dataframe
        df2: modified pandas dataframe
        provenance: boolean; write out a log of proposed changes

    Output:
    --------
        indices: list of lists--the row-column pairs of unequal cells
    '''

    if df1.shape != df2.shape:
        raise Exception("These dataframes don't have the same number of rows and columns")
    else:
        comparison_array = df1.fillna(0).values == df2.fillna(0).values
        indices = np.where(comparison_array == False)
        indices = np.dstack(indices)[0].tolist()

        if provenance:

            cols = ["original", "modified", "row", "column"]
            lst = []

            for pair in indices:

                original = df1.iloc[pair[0], pair[1]]
                modified = df2.iloc[pair[0], pair[1]]
                column = df1.columns[pair[1]]
                row = df1.index[pair[0]]

                lst.append([original, modified, row, column])

            provenance_df = pd.DataFrame(lst, columns=cols)
            currentDT = datetime.now()
            fname = "provenance_{}.csv".format(currentDT.strftime("%Y-%m-%d_%H:%M:%S"))
            fname_exists = os.path.isfile(fname)
            if fname_exists:
                i = 1
                while fname_exists:
                    i += 1
                    fname = "provenance_{}_{}.csv".format(currentDT.strftime("%Y-%m-%d_%H:%M:%S"), i)
                    fname_exists = os.path.isfile(fname)


            provenance_df.to_csv(fname, index=False, na_rep="NA")

        return(indices)

def is_nan(x):
    return (x is np.nan or x != x)
